==> localstack_ext-3.5.1.dev20240719061349.tar.gz: localstack_ext-3.5.1.dev20240719061349/localstack/pro/core/bootstrap/ftp_server.py <==
_B='UserDirectory'
_A=True
import glob,json,logging,os,subprocess,sys,time
from ftplib import FTP
from typing import TYPE_CHECKING
from localstack import config,constants
from localstack.aws.connect import connect_to
from localstack.pro.core import config as ext_config
from localstack.utils.files import load_file,new_tmp_dir,save_file
from localstack.utils.net import wait_for_port_open
from localstack.utils.run import ShellCommandThread
from localstack.utils.ssl import create_ssl_cert
from localstack.utils.threads import TMP_THREADS,FuncThread
if TYPE_CHECKING:from localstack.pro.core.services.transfer.models import User
LOG=logging.getLogger(__name__)
ROOT_USER='root','pass123'
FTP_USER_DEFAULT_PASSWD='12345'
FTP_USER_PERMISSIONS='elradfmwMT'
FTP_PASSIVE_PORTS=[config.EXTERNAL_SERVICE_PORTS_END-2,config.EXTERNAL_SERVICE_PORTS_END-1,config.EXTERNAL_SERVICE_PORTS_END]
USE_SUBPROCESS=_A
DIRECTORY_MAPPING={}
DIRECTORY_MAPPING_FILE='<data_dir>/ftp.user.dir.mapping.json'
def get_dir_mapping_key(username,server_port):return'{}:{}'.format(username,server_port)
def apply_patches():
	from pyftpdlib.handlers import FTPHandler as B,TLS_FTPHandler as A;C=A.proto_cmds.copy();C.update({'SITE ADDUSER':dict(perm='m',auth=_A,arg=_A,help='Syntax: SITE <SP> ADDUSER USERNAME PASSWORD HOME PRIVS <SP>.')})
	def D(self,file_path):
		C=file_path;A=get_dir_mapping_key(self.username,self.server.address[1]);E=get_directory_mapping();B=E.get(A,{})
		if not B:return
		D=B['HomeDirectory'];F=B[_B];A=C.replace('{}/'.format(F),'')
		with open(C)as G:H=connect_to().s3;H.put_object(Bucket=D,Key=A,Body=G.read());LOG.info('Received file via FTP -- target: s3://{}/{}'.format(D,A))
	def E(self,line):A,B,C,D=line.split(' ')[1:];self.authorizer.add_user(A,B,C,D);self.respond('201 Add User OK.')
	B.proto_cmds=C;A.proto_cmds=C;B.on_file_received=D;B.ftp_SITE_ADDUSER=E;A.on_file_received=D;A.ftp_SITE_ADDUSER=E
def add_ftp_user(user,server_port):C=server_port;B=user;A=FTP();A.connect(constants.LOCALHOST_HOSTNAME,port=C);A.login(ROOT_USER[0],ROOT_USER[1]);D=new_tmp_dir();A.sendcmd('SITE ADDUSER  {} {} {} {}'.format(B.user_name,FTP_USER_DEFAULT_PASSWD,D,FTP_USER_PERMISSIONS));A.quit();F=get_dir_mapping_key(B.user_name,C);E=B.get_directory_configuration();E.update({_B:D});set_directory_mapping(F,E)
def update_ftp_user(user,server_port):A=get_dir_mapping_key(user.user_name,server_port);set_directory_mapping(A,user.get_directory_configuration())
def start_ftp(port):
	G='PYTHONPATH';F='SERVICES';D=port
	if USE_SUBPROCESS:
		B=os.environ.get(F,'')
		if B and's3'not in B:B+=',s3'
		A=os.environ.get(G)or''
		if os.getcwd()not in A.split(':'):A='%s:%s'%(os.getcwd(),A)
		E=glob.glob('%s/.venv/lib/python*/site-packages'%os.getcwd())
		if E:A+=':%s'%':'.join(E)
		H={F:B,G:A};I='%s %s %s'%(sys.executable,__file__,D);C=ShellCommandThread(I,outfile=subprocess.PIPE,env_vars=H,quiet=False);C.start();time.sleep(2)
	else:C=do_start_ftp(D,asynchronous=_A)
	time.sleep(2);wait_for_port_open(D,retries=10,sleep_time=1.5);TMP_THREADS.append(C);return C
def do_start_ftp(port,asynchronous=_A):
	B=port;from pyftpdlib.authorizers import DummyAuthorizer as F;from pyftpdlib.handlers import TLS_FTPHandler as G;from pyftpdlib.servers import FTPServer as H;LOG.info('Starting (S)FTP server on port %s...'%B);apply_patches();C=F();I=new_tmp_dir();C.add_user(ROOT_USER[0],ROOT_USER[1],I,perm=FTP_USER_PERMISSIONS);J=new_tmp_dir();C.add_anonymous(J);A=G;K,L,L=create_ssl_cert();A.certfile=K;A.authorizer=C;A.passive_ports=FTP_PASSIVE_PORTS;A.masquerade_address=ext_config.LOCALHOST_IP
	def D(*E):
		try:C=H(('0.0.0.0',B),A);C.serve_forever()
		except Exception as D:LOG.info('Unable to run FTP server on port %s: %s'%(B,D));raise
	if asynchronous:E=FuncThread(D);E.start();return E
	return D()
def get_directory_mapping():
	A=DIRECTORY_MAPPING
	if USE_SUBPROCESS:B=get_directory_mapping_file();A=json.loads(load_file(B)or'{}')
	return A
def set_directory_mapping(key,value):
	A=value;B=get_directory_mapping();B[key]=A
	if USE_SUBPROCESS:C=get_directory_mapping_file();save_file(C,json.dumps(B))
	return A
def get_directory_mapping_file():return DIRECTORY_MAPPING_FILE.replace('<data_dir>',config.dirs.tmp)
def main():do_start_ftp(int(sys.argv[1]),asynchronous=False)
if __name__=='__main__':main()

==> localstack_ext-3.7.3.dev1.tar.gz: localstack_ext-3.7.3.dev1/localstack/pro/core/bootstrap/ftp_server.py <==
_B='UserDirectory'
_A=True
import glob,json,logging,os,subprocess,sys,time
from ftplib import FTP
from typing import TYPE_CHECKING
from localstack import config,constants
from localstack.aws.connect import connect_to
from localstack.pro.core import config as ext_config
from localstack.utils.files import load_file,new_tmp_dir,save_file
from localstack.utils.net import wait_for_port_open
from localstack.utils.run import ShellCommandThread
from localstack.utils.ssl import create_ssl_cert
from localstack.utils.threads import TMP_THREADS,FuncThread
if TYPE_CHECKING:from localstack.pro.core.services.transfer.models import User
LOG=logging.getLogger(__name__)
ROOT_USER='root','pass123'
FTP_USER_DEFAULT_PASSWD='12345'
FTP_USER_PERMISSIONS='elradfmwMT'
FTP_PASSIVE_PORTS=[config.EXTERNAL_SERVICE_PORTS_END-2,config.EXTERNAL_SERVICE_PORTS_END-1,config.EXTERNAL_SERVICE_PORTS_END]
USE_SUBPROCESS=_A
DIRECTORY_MAPPING={}
DIRECTORY_MAPPING_FILE='<data_dir>/ftp.user.dir.mapping.json'
def get_dir_mapping_key(username,server_port):return'{}:{}'.format(username,server_port)
def apply_patches():
	from pyftpdlib.handlers import FTPHandler as B,TLS_FTPHandler as A;C=A.proto_cmds.copy();C.update({'SITE ADDUSER':dict(perm='m',auth=_A,arg=_A,help='Syntax: SITE <SP> ADDUSER USERNAME PASSWORD HOME PRIVS <SP>.')})
	def D(self,file_path):
		C=file_path;A=get_dir_mapping_key(self.username,self.server.address[1]);E=get_directory_mapping();B=E.get(A,{})
		if not B:return
		D=B['HomeDirectory'];F=B[_B];A=C.replace('{}/'.format(F),'')
		with open(C)as G:H=connect_to().s3;H.put_object(Bucket=D,Key=A,Body=G.read());LOG.info('Received file via FTP -- target: s3://%s/%s',D,A)
	def E(self,line):A,B,C,D=line.split(' ')[1:];self.authorizer.add_user(A,B,C,D);self.respond('201 Add User OK.')
	B.proto_cmds=C;A.proto_cmds=C;B.on_file_received=D;B.ftp_SITE_ADDUSER=E;A.on_file_received=D;A.ftp_SITE_ADDUSER=E
def add_ftp_user(user,server_port):C=server_port;B=user;A=FTP();A.connect(constants.LOCALHOST_HOSTNAME,port=C);A.login(ROOT_USER[0],ROOT_USER[1]);D=new_tmp_dir();A.sendcmd('SITE ADDUSER  {} {} {} {}'.format(B.user_name,FTP_USER_DEFAULT_PASSWD,D,FTP_USER_PERMISSIONS));A.quit();F=get_dir_mapping_key(B.user_name,C);E=B.get_directory_configuration();E.update({_B:D});set_directory_mapping(F,E)
def update_ftp_user(user,server_port):A=get_dir_mapping_key(user.user_name,server_port);set_directory_mapping(A,user.get_directory_configuration())
def start_ftp(port):
	G='PYTHONPATH';F='SERVICES';D=port
	if USE_SUBPROCESS:
		B=os.environ.get(F,'')
		if B and's3'not in B:B+=',s3'
		A=os.environ.get(G)or''
		if os.getcwd()not in A.split(':'):A='%s:%s'%(os.getcwd(),A)
		E=glob.glob('%s/.venv/lib/python*/site-packages'%os.getcwd())
		if E:A+=':%s'%':'.join(E)
		H={F:B,G:A};I='%s %s %s'%(sys.executable,__file__,D);C=ShellCommandThread(I,outfile=subprocess.PIPE,env_vars=H,quiet=False);C.start();time.sleep(2)
	else:C=do_start_ftp(D,asynchronous=_A)
	time.sleep(2);wait_for_port_open(D,retries=10,sleep_time=1.5);TMP_THREADS.append(C);return C
def do_start_ftp(port,asynchronous=_A):
	B=port;from pyftpdlib.authorizers import DummyAuthorizer as F;from pyftpdlib.handlers import TLS_FTPHandler as G;from pyftpdlib.servers import FTPServer as H;LOG.info('Starting (S)FTP server on port %s...',B);apply_patches();C=F();I=new_tmp_dir();C.add_user(ROOT_USER[0],ROOT_USER[1],I,perm=FTP_USER_PERMISSIONS);J=new_tmp_dir();C.add_anonymous(J);A=G;K,L,L=create_ssl_cert();A.certfile=K;A.authorizer=C;A.passive_ports=FTP_PASSIVE_PORTS;A.masquerade_address=ext_config.LOCALHOST_IP
	def D(*E):
		try:C=H(('0.0.0.0',B),A);C.serve_forever()
		except Exception as D:LOG.info('Unable to run FTP server on port %s: %s',B,D);raise
	if asynchronous:E=FuncThread(D);E.start();return E
	return D()
def get_directory_mapping():
	A=DIRECTORY_MAPPING
	if USE_SUBPROCESS:B=get_directory_mapping_file();A=json.loads(load_file(B)or'{}')
	return A
def set_directory_mapping(key,value):
	A=value;B=get_directory_mapping();B[key]=A
	if USE_SUBPROCESS:C=get_directory_mapping_file();save_file(C,json.dumps(B))
	return A
def get_directory_mapping_file():return DIRECTORY_MAPPING_FILE.replace('<data_dir>',config.dirs.tmp)
def main():do_start_ftp(int(sys.argv[1]),asynchronous=False)
if __name__=='__main__':main()
